// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { __awaiter } from "tslib";
import { createPipelineFromOptions, isTokenCredential, signingPolicy, operationOptionsToRequestOptionsBase } from "@azure/core-http";
import { getTracer } from "@azure/core-tracing";
import { logger } from "./log";
import { SDK_VERSION } from "./constants";
import { KeyVaultClient } from "./generated/keyVaultClient";
import { challengeBasedAuthenticationPolicy } from "../../keyvault-common/src";
import { localSupportedAlgorithms, isLocallySupported } from "./localCryptography/algorithms";
import { LocalCryptographyClient } from "./localCryptographyClient";
import { LATEST_API_VERSION } from "./keysModels";
import { parseKeyVaultKeyId } from "./identifier";
/**
 * Checks whether a key can be used at that specific moment,
 * by comparing the current date with the bundle's notBefore and expires values.
 */
export function checkKeyValidity(keyId, keyBundle) {
    const attributes = (keyBundle === null || keyBundle === void 0 ? void 0 : keyBundle.attributes) || {};
    const { notBefore, expires } = attributes;
    const now = new Date();
    if (!keyId) {
        throw new Error("Only local cryptography operations can be performed on JsonWebKeys without kid");
    }
    if (notBefore && now < notBefore) {
        throw new Error(`Key ${keyId} can't be used before ${notBefore.toISOString()}`);
    }
    if (expires && now > expires) {
        throw new Error(`Key ${keyId} expired at ${expires.toISOString()}`);
    }
}
/**
 * A client used to perform cryptographic operations with Azure Key Vault keys.
 */
export class CryptographyClient {
    /**
     * Constructs a new instance of the Cryptography client for the given key
     *
     * Example usage:
     * ```ts
     * import { KeyClient, CryptographyClient } from "@azure/keyvault-keys";
     * import { DefaultAzureCredential } from "@azure/identity";
     *
     * let vaultUrl = `https://<MY KEYVAULT HERE>.vault.azure.net`;
     * let credentials = new DefaultAzureCredential();
     *
     * let keyClient = new KeyClient(vaultUrl, credentials);
     * let keyVaultKey = await keyClient.getKey("MyKey");
     *
     * let client = new CryptographyClient(keyVaultKey.id, credentials);
     * // or
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * ```
     * @param key The key to use during cryptography tasks. You can also pass the identifier of the key i.e its url here.
     * @param {TokenCredential} credential An object that implements the `TokenCredential` interface used to authenticate requests to the service. Use the @azure/identity package to create a credential that suits your needs.
     * @param {PipelineOptions} [pipelineOptions={}] Optional. Pipeline options used to configure Key Vault API requests.
     *                                                         Omit this parameter to use the default pipeline configuration.
     * @memberof CryptographyClient
     */
    constructor(key, credential, pipelineOptions = {}) {
        const libInfo = `azsdk-js-keyvault-keys/${SDK_VERSION}`;
        const userAgentOptions = pipelineOptions.userAgentOptions;
        pipelineOptions.userAgentOptions = Object.assign(Object.assign({}, pipelineOptions.userAgentOptions), { userAgentPrefix: userAgentOptions && userAgentOptions.userAgentPrefix
                ? `${userAgentOptions.userAgentPrefix} ${libInfo}`
                : libInfo });
        const authPolicy = isTokenCredential(credential)
            ? challengeBasedAuthenticationPolicy(credential)
            : signingPolicy(credential);
        const internalPipelineOptions = Object.assign(Object.assign({}, pipelineOptions), {
            loggingOptions: {
                logger: logger.info,
                logPolicyOptions: {
                    allowedHeaderNames: [
                        "x-ms-keyvault-region",
                        "x-ms-keyvault-network-info",
                        "x-ms-keyvault-service-version"
                    ]
                }
            }
        });
        const pipeline = createPipelineFromOptions(internalPipelineOptions, authPolicy);
        this.client = new KeyVaultClient(pipelineOptions.serviceVersion || LATEST_API_VERSION, pipeline);
        let parsed;
        if (typeof key === "string") {
            this.key = key;
            parsed = parseKeyVaultKeyId(this.key);
        }
        else if (key.key) {
            this.key = key.key;
            parsed = parseKeyVaultKeyId(this.key.kid);
        }
        else {
            throw new Error("The provided key is malformed as it does not have a value for the `key` property.");
        }
        if (parsed.name === "") {
            throw new Error("Could not find 'name' of key in key URL");
        }
        if (!parsed.version || parsed.version === "") {
            throw new Error("Could not find 'version' of key in key URL");
        }
        if (!parsed.vaultUrl || parsed.vaultUrl === "") {
            throw new Error("Could not find 'vaultUrl' of key in key URL");
        }
        this.vaultUrl = parsed.vaultUrl;
        this.name = parsed.name;
        this.version = parsed.version;
    }
    /**
     * @internal
     * @ignore
     * Retrieves the {@link JsonWebKey} from the Key Vault.
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.getKey();
     * ```
     * @param {GetKeyOptions} [options] Options for retrieving key.
     */
    getKey(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("getKey", requestOptions);
            if (typeof this.key === "string") {
                if (!this.name || this.name === "") {
                    throw new Error("getKey requires a key with a name");
                }
                const keyBundle = yield this.client.getKey(this.vaultUrl, this.name, options && options.version ? options.version : this.version ? this.version : "", this.setParentSpan(span, requestOptions));
                this.keyBundle = keyBundle;
                return keyBundle.key;
            }
            else {
                return this.key;
            }
        });
    }
    /**
     * Encrypts the given plaintext with the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.encrypt("RSA1_5", Buffer.from("My Message"));
     * ```
     * @param {EncryptionAlgorithm} algorithm The algorithm to use.
     * @param {Uint8Array} plaintext The text to encrypt.
     * @param {EncryptOptions} [options] Additional options.
     */
    encrypt(algorithm, plaintext, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const localCryptographyClient = yield this.getLocalCryptographyClient();
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("encrypt", requestOptions);
            yield this.checkPermissions("encrypt");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            if (localCryptographyClient && isLocallySupported(algorithm)) {
                try {
                    return localCryptographyClient.encrypt(algorithm, plaintext);
                }
                catch (e) {
                    if (e.name !== "LocalCryptographyUnsupportedError") {
                        span.end();
                        throw e;
                    }
                }
            }
            // Default to the service
            let result;
            try {
                result = yield this.client.encrypt(this.vaultUrl, this.name, this.version, algorithm, plaintext, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, algorithm, keyID: this.getKeyID() };
        });
    }
    /**
     * Decrypts the given ciphertext with the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.decrypt("RSA1_5", encryptedBuffer);
     * ```
     * @param {EncryptionAlgorithm} algorithm The algorithm to use.
     * @param {Uint8Array} ciphertext The text to decrypt.
     * @param {DecryptOptions} [options] Additional options.
     */
    decrypt(algorithm, ciphertext, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("decrypt", requestOptions);
            yield this.checkPermissions("decrypt");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            // Default to the service
            let result;
            try {
                result = yield this.client.decrypt(this.vaultUrl, this.name, this.version, algorithm, ciphertext, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, keyID: this.getKeyID(), algorithm };
        });
    }
    /**
     * Wraps the given key using the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.wrapKey("RSA1_5", keyToWrap);
     * ```
     * @param {KeyWrapAlgorithm} algorithm The encryption algorithm to use to wrap the given key.
     * @param {Uint8Array} key The key to wrap.
     * @param {WrapKeyOptions} [options] Additional options.
     */
    wrapKey(algorithm, key, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const localCryptographyClient = yield this.getLocalCryptographyClient();
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("decrypt", requestOptions);
            yield this.checkPermissions("wrapKey");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            if (localCryptographyClient && isLocallySupported(algorithm)) {
                try {
                    return localCryptographyClient.wrapKey(algorithm, key);
                }
                catch (e) {
                    if (e.name !== "LocalCryptographyUnsupportedError") {
                        span.end();
                        throw e;
                    }
                }
            }
            // Default to the service
            let result;
            try {
                result = yield this.client.wrapKey(this.vaultUrl, this.name, this.version, algorithm, key, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, algorithm, keyID: this.getKeyID() };
        });
    }
    /**
     * Unwraps the given wrapped key using the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.unwrapKey("RSA1_5", keyToUnwrap);
     * ```
     * @param {KeyWrapAlgorithm} algorithm The decryption algorithm to use to unwrap the key.
     * @param {Uint8Array} encryptedKey The encrypted key to unwrap.
     * @param {UnwrapKeyOptions} [options] Additional options.
     */
    unwrapKey(algorithm, encryptedKey, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("unwrapKey", requestOptions);
            yield this.checkPermissions("unwrapKey");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            // Default to the service
            let result;
            try {
                result = yield this.client.unwrapKey(this.vaultUrl, this.name, this.version, algorithm, encryptedKey, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, keyID: this.getKeyID() };
        });
    }
    /**
     * Cryptographically sign the digest of a message
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.sign("RS256", digest);
     * ```
     * @param {KeySignatureAlgorithm} algorithm The signing algorithm to use.
     * @param {Uint8Array} digest The digest of the data to sign.
     * @param {SignOptions} [options] Additional options.
     */
    sign(algorithm, digest, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("sign", requestOptions);
            yield this.checkPermissions("sign");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            let result;
            try {
                result = yield this.client.sign(this.vaultUrl, this.name, this.version, algorithm, digest, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, algorithm, keyID: this.getKeyID() };
        });
    }
    /**
     * Verify the signed message digest
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.verify("RS256", signedDigest, signature);
     * ```
     * @param {KeySignatureAlgorithm} algorithm The signing algorithm to use to verify with.
     * @param {Uint8Array} digest The digest to verify.
     * @param {Uint8Array} signature The signature to verify the digest against.
     * @param {VerifyOptions} [options] Additional options.
     */
    verify(algorithm, digest, signature, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("verify", requestOptions);
            yield this.checkPermissions("verify");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            let response;
            try {
                response = yield this.client.verify(this.vaultUrl, this.name, this.version, algorithm, digest, signature, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: response.value ? response.value : false, keyID: this.getKeyID() };
        });
    }
    /**
     * Cryptographically sign a block of data
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.signData("RS256", message);
     * ```
     * @param {KeySignatureAlgorithm} algorithm The signing algorithm to use.
     * @param {Uint8Array} data The data to sign.
     * @param {SignOptions} [options] Additional options.
     */
    signData(algorithm, data, options = {}) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("unwrapKey", requestOptions);
            yield this.checkPermissions("sign");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            if (!isLocallySupported(algorithm)) {
                throw new Error(`Unsupported algorithm ${algorithm}`);
            }
            const localAlgorithm = algorithm;
            // Not supported locally yet
            const createHash = (_a = localSupportedAlgorithms[localAlgorithm]) === null || _a === void 0 ? void 0 : _a.operations.createHash;
            const digest = yield createHash("", Buffer.from(data));
            // Default to the service
            let result;
            try {
                result = yield this.client.sign(this.vaultUrl, this.name, this.version, algorithm, digest, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.result, algorithm, keyID: this.getKeyID() };
        });
    }
    /**
     * Verify the signed block of data
     *
     * Example usage:
     * ```ts
     * let client = new CryptographyClient(keyVaultKey, credentials);
     * let result = await client.verifyData("RS256", signedMessage, signature);
     * ```
     * @param {KeySignatureAlgorithm} algorithm The algorithm to use to verify with.
     * @param {Uint8Array} data The signed block of data to verify.
     * @param {Uint8Array} signature The signature to verify the block against.
     * @param {VerifyOptions} [options] Additional options.
     */
    verifyData(algorithm, data, signature, options = {}) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const localCryptographyClient = yield this.getLocalCryptographyClient();
            const requestOptions = operationOptionsToRequestOptionsBase(options);
            const span = this.createSpan("decrypt", requestOptions);
            yield this.checkPermissions("verify");
            yield this.getLocalCryptographyClient();
            checkKeyValidity(this.getKeyID(), this.keyBundle);
            if (!isLocallySupported(algorithm)) {
                throw new Error(`Unsupported algorithm ${algorithm}`);
            }
            const localAlgorithm = algorithm;
            if (localCryptographyClient) {
                try {
                    return localCryptographyClient.verifyData(localAlgorithm, data, signature);
                }
                catch (e) {
                    if (e.name !== "LocalCryptographyUnsupportedError") {
                        span.end();
                        throw e;
                    }
                }
            }
            const createHash = (_a = localSupportedAlgorithms[localAlgorithm]) === null || _a === void 0 ? void 0 : _a.operations.createHash;
            const digest = yield createHash("", Buffer.from(data));
            // Default to the service
            let result;
            try {
                result = yield this.client.verify(this.vaultUrl, this.name, this.version, algorithm, digest, signature, this.setParentSpan(span, requestOptions));
            }
            finally {
                span.end();
            }
            return { result: result.value, keyID: this.getKeyID() };
        });
    }
    /**
     * @internal
     * @ignore
     * Attempts to retrieve the ID of the key.
     */
    getKeyID() {
        let kid;
        if (typeof this.key !== "string") {
            kid = this.key.kid;
        }
        else {
            kid = this.key;
        }
        return kid;
    }
    /**
     * Returns a promise that resolves once the Local Cryptography Client is available.
     */
    getLocalCryptographyClient() {
        if (!this._localCryptographyClientPromise) {
            this._localCryptographyClientPromise = this.configureLocalCryptographyClient();
        }
        return this._localCryptographyClientPromise;
    }
    configureLocalCryptographyClient() {
        return __awaiter(this, void 0, void 0, function* () {
            this.key = yield this.getKey();
            return new LocalCryptographyClient(this.key);
        });
    }
    /**
     * @internal
     * @ignore
     * Creates a span using the tracer that was set by the user.
     * @param {string} methodName The name of the method creating the span.
     * @param {RequestOptionsBase} [options] The options for the underlying HTTP request.
     */
    createSpan(methodName, requestOptions) {
        const tracer = getTracer();
        const span = tracer.startSpan(`CryptographyClient ${methodName}`, requestOptions && requestOptions.spanOptions);
        span.setAttribute("az.namespace", "Microsoft.KeyVault");
        return span;
    }
    /**
     * @internal
     * @ignore
     * Returns updated HTTP options with the given span as the parent of future spans,
     * if applicable.
     * @param {Span} span The span for the current operation.
     * @param {RequestOptionsBase} [options] The options for the underlying HTTP request.
     */
    setParentSpan(span, options = {}) {
        if (span.isRecording()) {
            const spanOptions = options.spanOptions || {};
            return Object.assign(Object.assign({}, options), { spanOptions: Object.assign(Object.assign({}, spanOptions), { parent: span.context(), attributes: Object.assign(Object.assign({}, spanOptions.attributes), { "az.namespace": "Microsoft.KeyVault" }) }) });
        }
        else {
            return options;
        }
    }
    /**
     * Checks whether the internal key can be used to execute a given operation, by the operation's name.
     * @param operation The name of the operation that is expected to be viable
     */
    checkPermissions(operation) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getLocalCryptographyClient();
            if (typeof this.key !== "string" && this.key.keyOps && !this.key.keyOps.includes(operation)) {
                throw new Error(`Operation ${operation} is not supported on key ${this.getKeyID()}`);
            }
        });
    }
}
//# sourceMappingURL=cryptographyClient.js.map