// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { __awaiter } from "tslib";
import { publicEncrypt, createVerify } from "crypto";
import * as constants from "constants";
import { isNode } from "@azure/core-http";
import { LocalCryptographyUnsupportedError } from "./models";
import { createHash } from "./hash";
/**
 * This file contains the implementation of local supported algorithms, which
 * consist of wrappers around methods already available as part of NodeJS.
 *
 * The goal of this file is to abstract away how we implement these algorithms,
 * since we might be using environment-specific code or dependencies.
 *
 * Some algorithms can't be implemented neither in the Browser or in Node
 * without third party dependencies or considerable development.
 *
 * We currently don't support any operation that requires the private data of a
 * key. Once we implement support for the private part of JWK keys and PEM keys,
 * we will be able to increase the support of our existing algorithms.
 */
/**
 * @internal
 * @ignore
 * The list of known assertions so far.
 * Assertions verify that the requirements to execute a local cryptography operation are met.
 */
export const assertions = {
    /**
     * Validates that the target local cryptography operation is allowed by the key's "keyOps" property.
     */
    keyOps(key, operationName) {
        if (key && key.keyOps && !key.keyOps.includes(operationName)) {
            throw new Error(`Key does not support the ${operationName} operation`);
        }
    },
    /**
     * Validates that the key type is "RSA". For operations on RSA keys.
     */
    rsa(key) {
        if (key && key.kty !== "RSA" && key.kty !== "RSA-HSM") {
            throw new Error("Key type does not match the algorithm RSA");
        }
    },
    /**
     * Passes only when the operation is being executed in NodeJS.
     * For operations that can only run in NodeJS.
     */
    nodeOnly() {
        if (!isNode) {
            throw new LocalCryptographyUnsupportedError("This operation is only available in NodeJS");
        }
    }
};
/**
 * pipeAssertions allows us to execute a sequence of assertions.
 * @param assertions One or more LocalAssertions
 */
const pipeAssertions = (...assertions) => (...params) => {
    for (const assertion of assertions) {
        assertion(...params);
    }
};
/**
 * Local support of the RSA1_5 algorithm.
 * We currently only support encrypting and wrapping keys with it.
 */
const RSA1_5 = {
    validate: pipeAssertions(assertions.keyOps, assertions.rsa, assertions.nodeOnly),
    operations: {
        encrypt(keyPEM, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return publicEncrypt({ key: keyPEM, padding: constants.RSA_PKCS1_PADDING }, data);
            });
        },
        wrapKey(keyPEM, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return publicEncrypt({ key: keyPEM, padding: constants.RSA_PKCS1_PADDING }, data);
            });
        }
    }
};
/**
 * Local support of the RSA-OAEP algorithm.
 * We currently only support encrypting and wrapping keys with it.
 */
const RSA_OAEP = {
    validate: pipeAssertions(assertions.keyOps, assertions.rsa, assertions.nodeOnly),
    operations: {
        encrypt(keyPEM, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return publicEncrypt(keyPEM, data);
            });
        },
        wrapKey(keyPEM, data) {
            return __awaiter(this, void 0, void 0, function* () {
                return publicEncrypt(keyPEM, data);
            });
        }
    }
};
/**
 * Since sign algorithms behave almost the same, we're making a generator to save up code.
 * We receive the sign algorithm, from the list of names in `SignAlgorithmName`,
 * then we generate a `LocalSupportedAlgorithm` that only create hashes and verifies signatures.
 * @param signAlgorithm
 */
const makeSigner = (signAlgorithm) => {
    return {
        validate: pipeAssertions(assertions.keyOps, assertions.nodeOnly),
        signAlgorithm,
        operations: {
            createHash(_keyPEM, data) {
                return __awaiter(this, void 0, void 0, function* () {
                    return createHash(signAlgorithm, data);
                });
            },
            verify(keyPEM, data, signature) {
                return __awaiter(this, void 0, void 0, function* () {
                    const verifier = createVerify(signAlgorithm);
                    verifier.update(data);
                    verifier.end();
                    return verifier.verify(keyPEM, signature);
                });
            }
        }
    };
};
/**
 * A plain object containing all of the locally supported algorithms.
 */
export const localSupportedAlgorithms = {
    RSA1_5,
    "RSA-OAEP": RSA_OAEP,
    PS256: makeSigner("SHA256"),
    RS256: makeSigner("SHA256"),
    PS384: makeSigner("SHA384"),
    RS384: makeSigner("SHA384"),
    PS512: makeSigner("SHA512"),
    RS512: makeSigner("SHA512")
};
/**
 * Checks whether a given algorithm name is supported or not.
 * @param algorithm string name of the algorithm
 */
export function isLocallySupported(algorithm) {
    return !!localSupportedAlgorithms[algorithm];
}
//# sourceMappingURL=algorithms.js.map