// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
import { __awaiter } from "tslib";
import { isNode } from "@azure/core-http";
import { LocalCryptographyUnsupportedError } from "./localCryptography/models";
import { runOperation } from "./localCryptography/runOperation";
/**
 * A client used to perform local cryptographic operations with JSON Web Keys.
 */
export class LocalCryptographyClient {
    /**
     * Constructs a new instance of the Local Cryptography client for the given key.
     *
     * Example usage:
     * ```ts
     * import { LocalCryptographyClient } from "@azure/keyvault-keys";
     *
     * const jsonWebKey: JsonWebKey = {
     *   // ...
     * };
     * const client = new LocalCryptographyClient(jsonWebKey);
     * ```
     * @param key The JsonWebKey to use during cryptography operations.
     * @memberof CryptographyClient
     */
    constructor(key) {
        this.key = key;
    }
    /**
     * Encrypts the given plaintext with the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new LocalCryptographyClient(jsonWebKey);
     * let result = await client.encrypt("RSA1_5", Buffer.from("My Message"));
     * ```
     * @param {LocalSupportedAlgorithmName} algorithm The algorithm to use.
     * @param {Uint8Array} plaintext The text to encrypt.
     */
    encrypt(algorithm, plaintext) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!isNode) {
                throw new LocalCryptographyUnsupportedError("Encryption is only available in NodeJS");
            }
            const result = (yield runOperation(this.key, "encrypt", algorithm, Buffer.from(plaintext)));
            const keyID = this.key.kid;
            return { result, algorithm: algorithm, keyID };
        });
    }
    /**
     * Wraps the given key using the specified cryptography algorithm
     *
     * Example usage:
     * ```ts
     * let client = new LocalCryptographyClient(jsonWebKey);
     * let result = await client.wrapKey("RSA1_5", keyToWrap);
     * ```
     * @param {LocalSupportedAlgorithmName} algorithm The encryption algorithm to use to wrap the given key.
     * @param {Uint8Array} key The key to wrap.
     * @param {EncryptOptions} [options] Additional options.
     */
    wrapKey(algorithm, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = (yield runOperation(this.key, "wrapKey", algorithm, Buffer.from(key)));
            const keyID = this.key.kid;
            return { result, algorithm: algorithm, keyID };
        });
    }
    /**
     * Verify the signed block of data
     *
     * Example usage:
     * ```ts
     * let client = new LocalCryptographyClient(jsonWebKey, credentials);
     * let result = await client.verifyData("RS256", signedMessage, signature);
     * ```
     * @param {KeySignatureAlgorithm} algorithm The algorithm to use to verify with.
     * @param {Uint8Array} data The signed block of data to verify.
     * @param {Uint8Array} signature The signature to verify the block against.
     * @param {EncryptOptions} [options] Additional options.
     */
    verifyData(algorithm, data, signature) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = (yield runOperation(this.key, "verify", algorithm, Buffer.from(data), Buffer.from(signature)));
            const keyID = this.key.kid;
            return { result, keyID };
        });
    }
}
//# sourceMappingURL=localCryptographyClient.js.map